// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of passive_data;

/// Encapsulating class for all Pds methods.
class Pds {
  /// Convert a dynamic type to a PDS.
  static dynamic fromDynamic(final dynamic value, [bool retry = true]) {
    // Check for num, bool, null, String
    if (value is num || value is bool || value == null || value is String) {
      return value;
    } else {
      // Check for List, Set, Map<String, Map<int
      if (value is List) {
        return fromList(value);
      } else if (value is Set) {
        return fromSet(value);
      } else if (value is Map) {
        return fromMap(value);
      } else if (retry) {
        // Fallback to `.toPds()`
        try {
          var passive = value.toPds();
          return fromDynamic(passive, false);
        } catch (e) {
          throw new ArgumentError.value(value, 'value',
              'cannot be converted to a passive data structure');
        }
      } else {
        throw new ArgumentError.value(
            value, 'value', 'cannot be converted to a passive data structure');
      }
    }
  }

  /// Convert a List to a PDS.
  static List fromList(final List list) {
    return new List.generate(list.length, (int i) => fromDynamic(list[i]),
        growable: false);
  }

  /// Convert a Set to a PDS.
  static List fromSet(final Set theset) {
    return fromList(theset.toList(growable: false));
  }

  /// Convert a String indexed Map to a PDS.
  static Map<String, dynamic> fromMap(final Map map) {
    return new Map<String, dynamic>.fromIterable(map.keys,
        key: (key) => key is String ? key : key.toString(),
        value: (key) => fromDynamic(map[key]));
  }
}
